'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _underscore = require('underscore');

var _underscore2 = _interopRequireDefault(_underscore);

var _dotObject = require('dot-object');

var _dotObject2 = _interopRequireDefault(_dotObject);

var _FieldType = require('./FieldType');

var _types = require('./types');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var propTypes = {
  /**
   * The label of the field.
   */
  label: _react2.default.PropTypes.any,

  /**
   * The name of the field in the object.
   */
  fieldName: _react2.default.PropTypes.string.isRequired,

  /**
   * Should show label
   */
  showLabel: _react2.default.PropTypes.bool,

  /**
   * Use hint instead of label.
   */
  useHint: _react2.default.PropTypes.bool,

  /**
   * The field should be read only mode.
   */
  disabled: _react2.default.PropTypes.bool,

  /**
   * The type of the input. It can be a component
   */
  type: _react2.default.PropTypes.any,

  /**
   * Pass a error message
   */
  errorMessage: _react2.default.PropTypes.string
};

var defaultProps = {
  showLabel: true,
  useHint: false,
  disabled: false
};

var contextTypes = {
  schema: _react2.default.PropTypes.object,
  doc: _react2.default.PropTypes.object,
  onChange: _react2.default.PropTypes.func.isRequired,
  errorMessages: _react2.default.PropTypes.object,
  form: _react2.default.PropTypes.any.isRequired,
  parentFieldName: _react2.default.PropTypes.string
};

var Field = function (_React$Component) {
  _inherits(Field, _React$Component);

  function Field(props) {
    _classCallCheck(this, Field);

    /* if (!this.context.schema && !props.type) {
      throw new Error(`You must set the type for the field "${props.fieldName}" or pass a schema to the form`)
    } */

    var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(Field).call(this, props));

    _this.onChange = _this.onChange.bind(_this);
    return _this;
  }

  _createClass(Field, [{
    key: 'componentDidMount',
    value: function componentDidMount() {
      this.registerField();
    }
  }, {
    key: 'componentWillReceiveProps',
    value: function componentWillReceiveProps(nextProps) {
      if (!_underscore2.default.isEqual(this.props, nextProps)) {
        this.unregisterField();
      }
    }
  }, {
    key: 'componentDidUpdate',
    value: function componentDidUpdate(prevProps) {
      if (!_underscore2.default.isEqual(prevProps, this.props)) {
        this.registerField();
      }
    }
  }, {
    key: 'componentWillUnmount',
    value: function componentWillUnmount() {
      this.unregisterField();
    }
  }, {
    key: 'unregisterField',
    value: function unregisterField() {
      this.context.form.unregisterComponent(this.getFieldName());
    }
  }, {
    key: 'registerField',
    value: function registerField() {
      this.context.form.registerComponent({
        field: this.getFieldName(),
        component: this.element
      });
    }
  }, {
    key: 'getFieldName',
    value: function getFieldName() {
      if (this.context.parentFieldName) {
        return this.context.parentFieldName + '.' + this.props.fieldName;
      } else {
        return this.props.fieldName;
      }
    }
  }, {
    key: 'onChange',
    value: function onChange(value) {
      this.context.onChange(this.getFieldName(), value);
    }
  }, {
    key: 'getSchema',
    value: function getSchema() {
      return this.context.schema;
    }
  }, {
    key: 'getFieldSchema',
    value: function getFieldSchema() {
      return this.getSchema() ? this.getSchema().schema(this.getFieldName()) : null;
    }
  }, {
    key: 'getLabel',
    value: function getLabel() {
      if (_underscore2.default.has(this.props, 'label')) {
        return this.props.label;
      } else if (this.getSchema()) {
        return this.getSchema().label(this.getFieldName());
      } else {
        return '';
      }
    }
  }, {
    key: 'getComponent',
    value: function getComponent() {
      if (_underscore2.default.isString(this.props.type)) {
        return (0, _types.getFieldType)(this.props.type, this.props.fieldName).component;
      } else if (this.props.type) {
        return this.props.type;
      } else {
        return (0, _types.getFieldComponent)({
          fieldName: this.getFieldName(),
          schema: this.getSchema()
        });
      }
    }
  }, {
    key: 'getValue',
    value: function getValue() {
      return this.context.doc ? _dotObject2.default.pick(this.getFieldName(), this.context.doc) : undefined;
    }
  }, {
    key: 'getErrorMessage',
    value: function getErrorMessage() {
      var errorMessages = this.context.errorMessages || {};
      return this.props.errorMessage || errorMessages[this.getFieldName()];
    }
  }, {
    key: 'getChildProps',
    value: function getChildProps() {
      var typeName = this.props.type;
      if (!typeName) {
        typeName = (0, _types.getFieldTypeName)({ fieldName: this.getFieldName(), fieldSchema: this.getFieldSchema(), schema: this.getSchema() });
      }

      /**
       * This gets the props that are defined in the propTypes of the registered component.
       */
      var fieldComponent = this.getComponent();
      var propOptions = _underscore2.default.omit(this.props, _underscore2.default.keys(propTypes));
      var schemaOptions = this.getFieldSchema() && (this.getFieldSchema().srf || this.getFieldSchema().mrf) || {};
      var totalOptions = _underscore2.default.extend(schemaOptions, propOptions);
      var allowedKeys = _underscore2.default.keys(_extends({}, _FieldType.propTypes, fieldComponent.propTypes));
      var onlyAllowedOptions = _underscore2.default.pick(totalOptions, allowedKeys);

      /**
       * Options that are not registered in the propTypes are passed separatly.
       * This options are in the variable this.passProps of the component, they should be
       * passed to the main component of it.
       */
      allowedKeys.push('type');
      var notDefinedOptions = _underscore2.default.omit(totalOptions, allowedKeys);

      var props = _extends({
        value: this.getValue(),
        label: this.props.showLabel ? this.getLabel() : null,
        useHint: this.props.useHint,
        onChange: this.onChange,
        errorMessage: this.getErrorMessage(),
        fieldSchema: this.getFieldSchema(),
        fieldName: this.getFieldName(),
        schema: this.getSchema(),
        form: this.context.form,
        disabled: this.props.disabled,
        passProps: notDefinedOptions,
        ref: 'input'
      }, onlyAllowedOptions);

      return props;
    }
  }, {
    key: 'render',
    value: function render() {
      var component = this.getComponent();
      this.element = _react2.default.createElement(component, this.getChildProps());
      return this.element;
    }
  }]);

  return Field;
}(_react2.default.Component);

exports.default = Field;


Field.propTypes = propTypes;
Field.defaultProps = defaultProps;
Field.contextTypes = contextTypes;